<?php
// $Id: relations.pages.inc,v 1.2 2008/08/29 15:25:00 arto Exp $

//////////////////////////////////////////////////////////////////////////////
// Node relationship tab

function relations_node_title_autocomplete($node, $string = '') {
  // TODO: filter out already related nodes.
  $matches = array();
  if (!empty($string)) {
    $result = db_query_range("SELECT nid, title FROM {node} WHERE nid != %d AND LOWER(title) LIKE LOWER('%s%%')", $node->nid, $string, 0, 10); // FIXME
    while ($node = db_fetch_object($result)) {
      $matches[$node->title] = check_plain($node->title);
    }
  }
  drupal_json($matches);
}

function relations_node_title_lookup($title) {
  return node_load(array('title' => $title));
}

function relations_node_tab($form_state, $node) {
  $form = array();
  $form['title'] = array('#type' => 'textfield', '#size' => 32, '#maxlength' => 255, '#autocomplete_path' => 'node/'. $node->nid .'/relations/autocomplete');
  $form['submit'] = array('#type' => 'submit', '#value' => t('Add relationship'));
  $form['nid'] = array('#type' => 'hidden', '#value' => $node->nid);
  return $form;
}

function relations_node_tab_validate($form, &$form_state) {
  extract($form_state['values'], EXTR_SKIP | EXTR_REFS);

  if (!($node = relations_node_title_lookup($title))) {
    form_set_error('title', t('No matching content titled %title found.', array('%title' => $title)));
  }
}

function relations_node_tab_submit($form, &$form_state) {
  extract($form_state['values'], EXTR_SKIP | EXTR_REFS);

  if ($node2 = relations_node_title_lookup($title)) {
    relations_api_create($nid, $node2->nid);

    watchdog('relations', 'Created relationship from node #%nid1 to #%nid2.', array('%nid1' => $nid, '%nid2' => $node2->nid), WATCHDOG_NOTICE, l(t('view'), 'node/'. $nid .'/relations'));
    drupal_set_message(t('The relationship to <a href="@url">%title</a> was created.', array('%title' => $title, '@url' => url('node/'. $node2->nid))));
    $form_state['redirect'] = 'node/'. $nid .'/relations';
  }
}

function relations_node_delete($form_state, $node1, $node2) {
  $form['nid1'] = array('#type' => 'value', '#value' => $node1->nid);
  $form['nid2'] = array('#type' => 'value', '#value' => $node2->nid);

  return confirm_form($form,
    t('Are you sure you want to delete the relationship to <a href="@url">%title</a>?', array('%title' => $node2->title, '@url' => url('node/'. $node2->nid))),
    isset($_GET['destination']) ? $_GET['destination'] : 'node/'. $node1->nid .'/relations',
    t('This action cannot be undone.'));
}

function relations_node_delete_submit($form, &$form_state) {
  extract($form_state['values'], EXTR_SKIP | EXTR_REFS);

  if ($form_state['values']['confirm']) {
    relations_api_delete($nid1, $nid2);

    watchdog('relations', 'Deleted relationship from node #%nid1 to #%nid2.', array('%nid1' => $nid1, '%nid2' => $nid2), WATCHDOG_NOTICE, l(t('view'), 'node/'. $nid1 .'/relations'));
    $node2 = node_load($nid2);
    drupal_set_message(t('The relationship to <a href="@url">%title</a> has been deleted.', array('%title' => ($node2 ? $node2->title : '#'. $nid2), '@url' => url('node/'. $nid2))));
  }

  $form_state['redirect'] = 'node/'. $nid1 .'/relations';
}

//////////////////////////////////////////////////////////////////////////////
// Node theming

function theme_relations_node_tab($form) {
  global $user;
  $manage = user_access('manage node relations');
  drupal_add_css(drupal_get_path('module', 'relations') .'/relations.css', 'module', 'all', FALSE);

  $head = array(t('Relationship'), array('data' => t('Operations'), 'colspan' => 1));
  $rows = array();

  foreach (relations_api_lookup($form['nid']['#value'], array('user' => $user)) as $related) {
    $rows[] = array(
      l($related->title, 'node/'. $related->nid),
      $manage ? l('delete relationship', 'node/'. $form['nid']['#value'] .'/relations/delete/'. $related->nid) : '',
    );
  }

  if (empty($rows)) {
    //$rows[] = array(array('data' => t('No relationships at present.'), 'colspan' => 2));
  }

  $rows[] = array(drupal_render($form['title']), array('data' => drupal_render($form['submit']), 'colspan' => 2));

  return drupal_render($form) . theme('table', $head, $rows, array('class' => 'relations'));
}

//////////////////////////////////////////////////////////////////////////////
// Block theming

function theme_relations_node_block($node) {
  global $user;
  drupal_add_css(drupal_get_path('module', 'relations') .'/relations.css', 'module', 'all', FALSE);

  return theme('relations_node_list', relations_api_lookup($node->nid, array('user' => $user)));
}

function theme_relations_node_list($nodes) {
  if (!empty($nodes)) {
    $output = '<div class="item-list">';
    $output .= '<ul>';
    foreach ($nodes as $node) {
      $output .= '<li>'. l($node->title, 'node/'. $node->nid) .'</li>';
    }
    $output .= '</ul>';
    $output .= '</div>';
    return $output;
  }
}
